/* ----------------------------------------------------------------------------
 * rishu_proxy.cpp
 *   里珠/Proxy のテスト.
 * ----------------------------------------------------------------------------
 * Mastering programed by YAMASHINA Hio
 *
 * Copyright 2002 YMIRLINK,Inc.
 * ----------------------------------------------------------------------------
 * $Id: test.c,v 1.3 2002/11/09 13:38:30 hio Exp $
 * ----------------------------------------------------------------------------
 */

/* #define TEST_DIRECT */

#include <windows.h>
#ifdef TEST_DIRECT
#define SHIORI_API_IMPLEMENTS
#endif
#include "shiori.h"
#include <stdio.h>
#include <direct.h> /* getcwd */

#ifdef TEST_DIRECT
/* ----------------------------------------------------------------------------
 * DLL にしないで obj を直接つなげるなら, 
 * 擬似的に DLL のロードを行うのでその前方宣言.
 */
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
		       );

#endif

static LPTSTR getLastErrorMessage(void);
static int test_load();
static int test_request();
static int test_unload();

/* ----------------------------------------------------------------------------
 * main 関数
 */
int main(int argc, char** argv)
{
  return test_load() || test_request() || test_unload();
}

/* ----------------------------------------------------------------------------
 * load の実行
 */
static char loaddir[256];
static const int loaddir_size = 256;
static int test_load()
{
  int len;
  HGLOBAL data;
  long data_len;
  BOOL res;
  
#ifdef TEST_DIRECT
  /* Dllの擬似ロード */
  res = DllMain(GetModuleHandle(NULL),DLL_PROCESS_ATTACH,0);
  if( !res )
  {
    fprintf(stderr,"DllMain returns FALSE\n");
    return 1;
  }
#endif
  
  /* ディレクトリの設定 */
#if 0
  ptr = _getcwd(cwd,255);
  if( ptr==NULL )
  {
    perror("getcwd failed");
    return 1;
  }
  len = strlen(ptr);
  if( ptr[len-1]!='\\' )
  {
    ptr[len++] = '\\';
  }
#endif
  
  printf("loaddir> ");
  fflush(stdout);
  if( !fgets(loaddir,loaddir_size,stdin) )
  {
    perror("fgets failed");
    return 1;
  }
  len = strlen(loaddir);
  if( len>0 && loaddir[len-1]=='\n' )
  {
    --len;
  }
  if( len>0 && loaddir[len-1]=='\r' )
  {
    --len;
  }
  
  /* ディレクトリ位置を HGLOBAL に変換. */
  data_len = len;
  data = GlobalAlloc(GMEM_FIXED,data_len);
  if( !data )
  {
    LPTSTR cause = getLastErrorMessage();
    fprintf(stderr,"GlovalAlloc failed: %s\n",cause);
    LocalFree(cause);
    return 1;
  }
  CopyMemory((LPVOID)data,loaddir,data_len);
  
  res = load(data,data_len);
  if( !res )
  {
    fprintf(stderr, "Shiori.load returns FALSE\n");
    return 1;
  }
  
  fprintf(stderr, "Shiori.load returns TRUE\n");
  return 0;
}

/* ----------------------------------------------------------------------------
 * reqeust の実行
 */
static char tmpbuf[256];
static const int tmpbuf_size = 256;
static int test_request()
{
  typedef struct Str
  {
    HGLOBAL hGlobal;
    DWORD len;
    DWORD size;
  } Str;
  Str str;
  
  str.hGlobal = GlobalAlloc(GMEM_FIXED,1024);
  if( str.hGlobal==NULL )
  {
    LPTSTR cause = getLastErrorMessage();
    fprintf(stderr,"GlovalAlloc failed: %s\n",cause);
    LocalFree(cause);
    return 1;
  }
  str.len = 0;
  str.size = GlobalSize(str.hGlobal);
  printf("<wait>\n");
  
  while( fgets(tmpbuf,tmpbuf_size,stdin) )
  {
    int len = strlen(tmpbuf);
    if( len>0 && tmpbuf[len-1]=='\n' ) --len;
    if( len>0 && tmpbuf[len-1]=='\r' ) --len;
    tmpbuf[len++] = '\r';
    tmpbuf[len++] = '\n';
    if( len+str.len>str.size )
    {
      str.hGlobal = GlobalReAlloc(str.hGlobal,len+str.len,0);
      if( str.hGlobal==NULL )
      {
	LPTSTR cause = getLastErrorMessage();
	fprintf(stderr,"GlovalAlloc failed: %s\n",cause);
	LocalFree(cause);
	return 1;
      }
      str.size = GlobalSize(str.hGlobal);
    }
    CopyMemory((LPBYTE)str.hGlobal+str.len,tmpbuf,len);
    str.len += len;
    if( len==2 )
    {
      str.hGlobal = request(str.hGlobal,(long*)&str.len);
      if( str.hGlobal==NULL )
      {
	printf("request returns NULL\n");
	return 1;
      }
      printf("<< test_request >>\n"
	     "[result of request]\n"
	     "[hGlobal==0x%08x] [len==%d]\n"
	     "[[\n"
	     "%.*s"
	     "]]\n"
	     "--------\n",
	     str.hGlobal, str.len,
	     str.len,str.hGlobal
	     );
      str.size = GlobalSize(str.hGlobal);
      str.len = 0;
      printf("<wait>\n");
    }
  }
  return 0;
}

/* ----------------------------------------------------------------------------
 * unload の実行
 */
static int test_unload()
{
  BOOL res;
  
  res = unload();
  if( !res )
  {
    fprintf(stderr, "Shiori.unload returns FALSE\n");
    return 1;
  }
  
  fprintf(stderr, "Shiori.unload returns TRUE\n");
  return 0;
}

/* ----------------------------------------------------------------------------
 * LPTSTR message = getLastErrorMessage();
 *   エラーメッセージの取得
 *   取得したメッセージは LocalFree で解放してね☆
 */
static LPTSTR getLastErrorMessage(void)
{
  LPVOID lpMessage;
  FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER
		 | FORMAT_MESSAGE_FROM_SYSTEM
		 | FORMAT_MESSAGE_IGNORE_INSERTS,
		 NULL,
		 GetLastError(),
		 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), /* 既定の言語 */
		 (LPTSTR)&lpMessage,
		 0,
		 NULL
		 );
  return lpMessage;
}
